
# Builds a command line string for clang++
def clang_compiler_build_str(out_name, files, inc_dirs, lib_dirs, libs, config, ex_linker_flags, ex_build_flags)

	# Compiler
	cc = "clang++"

	# Config
	if config == :debug
		config = "-g -O0"
	elsif config == :release
		config = "-O2"
	end

	# Files
	if files.class == Array then files = files.join(" ") end

	if files == :not_set
		return "echo \"No Files Given\""
	end

	# Inc Dirs
	if inc_dirs.class == Array then inc_dirs = inc_dirs.join(" -I") end
	if inc_dirs.class == String then inc_dirs = "-I" + inc_dirs end
	if inc_dirs == :not_set then inc_dirs = "" end

	# Lib Dirs
	if lib_dirs.class == Array then lib_dirs = lib_dirs.join(" -L") end
	if lib_dirs.class == String then lib_dirs = "-L" + lib_dirs end
	if lib_dirs == :not_set then lib_dirs = "" end

	# Libs
	if(lib_dirs != :not_set)
		if libs.class == Array then libs = libs.join(" -l") end
		if libs.class == String then libs = "-l" + libs end
	else
		libs = ""
	end


	#Extras
	if ex_linker_flags.class == Array then ex_linker_flags = ex_linker_flags.join(" ") end
	if ex_build_flags.class == Array then ex_build_flags = ex_build_flags.join(" ") end


	# Output
	build = "-o #{out_name}"

	# Other Stuff
	cpp_standard = "-std=c++11"

	framework_dir = "" #"-F /Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX10.10.sdk/System/Library/Frameworks/"

	# Build cmd string
	cmd_string = "#{cc} #{files} #{inc_dirs} #{lib_dirs} #{framework_dir} #{libs} #{ex_linker_flags} #{ex_build_flags} #{config} #{cpp_standard} #{build}"

	return cmd_string
end

$build_list = []

class CppBuild

	attr_accessor :name, :compiler, :files, :config, :inc_dirs, :lib_dirs, :libs, :arch, :defines, :out_dir
	attr_accessor :extra_linker_flags
	attr_accessor :extra_build_flags

	def initialize()
		@name = "cpp_build"
		@compiler = :clang
		@files = :not_set
		@config = :not_set
		@arch = :not_set
		@defines = :not_set
		@out_dir = "./"
	end


	def self.settings(&block)
		# Override the defaults.
	end


	def self.build(build_name, &block)

		build = CppBuild.new
		build.name = build_name.to_s
	    build.instance_eval(&block)


		if ARGV[0]
			unless ARGV[0] == build_name.to_s then return end
		else
			puts "#{build.name} - #{build.compiler.to_s}, #{build.config.to_s}, #{build.files.length.to_s} files"

			return
		end

	    if build.compiler == :clang
	    	compiler_cmd = clang_compiler_build_str(build.out_dir + build.name, build.files, build.inc_dirs, build.lib_dirs, build.libs, build.config, build.extra_linker_flags, build.extra_build_flags)
	    else
	    	compiler_cmd = "echo \"Compiler not supported.\""
	    end

	    puts "#{compiler_cmd}"

	    `#{compiler_cmd}`

	    # Hiding output
	    # if ARGV[1] == "-r"
	    # 	`./#{build.name}`
	    # end
	end


	def self.list()

	end

end

puts Dir.pwd

# Process arguments
ARGV.each do |a|

	# -r --run
	# -d <name> --define <name>
	# -cd --config_debug
	# -cr --config_release
	# -cc <name> --compiler <name>
	# -o <name> --output <name>
end



